
#include "p18f26k22.h"


extern unsigned char  donnee,status;	
extern unsigned char sec, min, hr, week_day, day, mn, year;


 char bcd2bin(char bcd_val)                                             
{
return ((bcd_val & 0x0f) + (((bcd_val >> 4) & 0x07) * 10));
}
 
 char bin2bcd(unsigned char binary_val) // value 00 to 99.
{
    return (((binary_val / 10) << 4)|((binary_val % 10))) ;
}
 
  
void I2CACK(unsigned char ackbit)
{
	
	if(ackbit)
	{
	SSP1CON2bits.ACKDT = 1;
	}
	else
	{
	SSP1CON2bits.ACKDT = 0;
	}

SSP1CON2bits.ACKEN = 1;
 while(SSP1CON2bits.ACKEN);
}
 
 
 
 void I2CIdle(void)
{
	while((SSP1STAT & 0x04) | (SSPCON2 & 0x1F));
}
 


void I2CStart(void)
{
	I2CIdle();
	SSPCON2bits.SEN = 1;
	while (SSPCON2bits.SEN);
}

void I2CRestart(void)
{
	I2CIdle();
	SSPCON2bits.RSEN = 1;
	while (SSPCON2bits.RSEN);
} 
 
 

void I2CStop(void)
{
	I2CIdle();
	SSPCON2bits.PEN = 1;
	while (SSPCON2bits.PEN);
}

void writeI2C(unsigned char data)
{
	PIR1bits.SSP1IF = 0;
	SSPBUF = data;
	while(!PIR1bits.SSP1IF);

}

char Read_I2C(char ack)
{
	I2CIdle();
	SSP1CON2bits.RCEN = 1;
	while(SSP1CON2bits.RCEN);
	while(!SSP1STATbits.BF);
	I2CACK(ack);
	return SSPBUF;
}

void ds1307_set_time(char hr, char min, char sec)
{
	sec &= 0x7F;
	min &= 0x7F;
	hr &= (0x7F|0x40);  //format 24H
	I2CStart();
	writeI2C(0XD0);           // I2C write address
	writeI2C(0x00);            		// WRT at REG 0 - Seconds
	writeI2C(bin2bcd(sec));      	// REG 0
	writeI2C(bin2bcd(min));      	// REG 1
	writeI2C(bin2bcd(hr ));      		 // REG 2 
	I2CStop();
}

void ds1307_set_date(char week_day,char day, char mn, char year )
{
	I2CStart();
	writeI2C(0XD0);    // I2C write address
	writeI2C(0x03);                      // Start at REG 3 - weekDay
	writeI2C(bin2bcd(week_day));
	writeI2C(bin2bcd(day));
	writeI2C(bin2bcd(mn));
	writeI2C(bin2bcd(year));
	writeI2C(0x10);                      // REG 7 - enable SQUAREW output pin
	I2CStop();
}

void ds1307_get_time(void)
{
	I2CStart();
	writeI2C(0XD0);
	writeI2C(0x00);                        // Start at REG 0 - Secondes
	I2CRestart();
	writeI2C(0XD1);
	sec = bcd2bin(Read_I2C(0));
	min = bcd2bin(Read_I2C(0));
	hr  = bcd2bin(Read_I2C(1));
	I2CStop();
} 


void ds1307_get_date(void)
{
	I2CStart();
	writeI2C(0XD0);
	writeI2C(0x03);                         // Start at REG 3 - Day of week
	I2CRestart();
	writeI2C(0XD1);
	// lecture DS1307 retourne (une valeur binaire codee BCD
	// ex retourne 19 ou 0x13   l'an 13 
	week_day  = bcd2bin(Read_I2C(0) );   // REG 3
	day = bcd2bin(Read_I2C(0) );   // REG 4
	mn  = bcd2bin(Read_I2C(0) );   // REG 5
	year = bcd2bin(Read_I2C(1));    // REG 6
	I2CStop();
}

